unit MFColorPicker;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,stdctrls,IniFiles;

type
  TColumnsX = (columns2,columns4,columns8);

type TCellClickEventX = procedure (Index : Integer;Button : TMouseButton;
                                  Color : TColor) of object;

type
 TCellX = class(TCustomPanel)
 private
  CellIndex : Integer;
 protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer); override;
 public
  Constructor Create(AOwner : TComponent); override;
end;

type
  TCustomPaletteX = class(TCustomPanel)
  private
    FOnCellClick : TCellClickEventX;
    FColumns : TColumnsX;
    CellWidth,CellHeight : Integer;
    AdjustWidth,AdjustHeight : Integer;
    Cells : array[0..15] of TCellX;
    Colors : array[0..15] of TColor;
    IniFile : TIniFile;
    procedure SetColumns(Num : TColumnsX);
 
  protected
    procedure PositionCells;
    procedure Resize; override;
  public
    FCellClicked : Integer;
    constructor Create(AOwner : TComponent); override;
    procedure SetStandardColors;
    procedure SetCellColor(index : integer;Color : TColor);
    function  GetCellColor(index : integer) : TColor;
    procedure SaveToFile(name : String);
    procedure LoadFromFile(name : string);
    procedure DoCellClick(Button : TMouseButton;Index : Integer;Color : TColor);
  published
    property OnResize;
    Property Enabled;
    Property Visible;
    property Columns : TColumnsX read FColumns write SetColumns default columns4;
    property OnCellClick : TCellClickEventX read FOnCellClick write FOnCellClick;
  end;

type
 TBtnSelectX = procedure(Color : TColor) of object;

type
  TColorPanelX = class(TCustomPanel)
  private
    Image : TImage;
    FRightSelect,FLeftSelect,FChange : TBtnSelectX;
    ColorSelected : TColor;
  protected
    procedure DoSelect(Sender: TObject;Button: TMouseButton;
                               Shift: TShiftState; X, Y: Integer);
    procedure DoChange(Sender: TObject; Shift: TShiftState;X, Y: Integer);
  public
    constructor Create(AOwner : TComponent); override;
  published
    Property Enabled;
    Property Visible;
    property OnLeftSelect : TBtnSelectX read FLeftSelect write FLeftSelect;
    property OnRightSelect : TBtnSelectX read FRightSelect write FRightSelect;
    property OnNewColor : TBtnSelectX read FChange write FChange;
  end;


//------- Color Picker Type Def ----------------------------------------


type
 TBtnSelect = procedure(Sender : TObject) of object;

type
 TNewColor = procedure of Object;

type
 TNewPaletteColor = procedure(Index : Integer; newColor : TColor) of Object;

type
  TColorPicker = class(TCustomPanel)
  private
    Back,Current,Prime,Second : TPanel;
    Group1,Group2 : TGroupBox;
    ColorBox : TColorPanelX;
    FPrimeColor,FSecondColor : TColor;
    Palette : TCustomPaletteX;
    BtnSave,BtnLoad : TButton;
    FSaveSelect,FLoadSelect : TBtnSelect;
    NewPrime,NewSecond : TNewColor;
    NewPalette : TNewPaletteColor;
  protected
    procedure Resize; override;
    procedure CurrentChange(aColor : TColor);
    procedure OnPaletteClick(Index : Integer;Button : TMouseButton;Color : TColor);
    procedure ButtonClick(Sender : TObject);
    procedure PrimaryColorChange(aColor : TColor);
    procedure SecondaryColorChange(aColor : TColor);
  public
    constructor Create(AOwner : TComponent); override;
    procedure SavePalette(name : string);
    procedure LoadPalette(name : string);
    Function  GetPaletteColor(Index : Integer) : TColor;
    Procedure SetPaletteColor(Index : Integer;aColor : TColor);
    Procedure SetStandardPalette;
  published
    Property Enabled;
    Property Visible;
    property OnSavePalette : TBtnSelect read FSaveSelect write FSaveSelect;
    property OnLoadPalette : TBtnSelect read FLoadSelect write FLoadSelect;
    Property PrimaryColor : TColor read FPrimeColor write PrimaryColorChange;
    Property SecondaryColor : TColor read FSecondColor write SecondaryColorChange;
    Property OnNewPrimary  : TNewColor read NewPrime write NewPrime;
    Property OnNewSecondary : TNewColor read NewSecond write NewSecond;
    Property OnPaletteChange : TNewPaletteColor read NewPalette write NewPalette;
  end;

procedure Register;

implementation

{$R MFColorPicker}
const
 TheCursor = 5002;

procedure Register;
begin
  RegisterComponents('MicroFrame', [TColorPicker]);
end;

//------ TCustomPaletteX Code ---------------------------------------------

constructor TCellX.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 Width := 10;
 Height := 10;
 BevelInner := bvLowered;
 BorderStyle := bsSingle;
 Cursor := TheCursor;

end;

procedure TCellX.MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer);
begin
 (Parent as TCustomPaletteX).FCellClicked := CellIndex;
end;

procedure TCellX.MouseUp(Button: TMouseButton; Shift: TShiftState;
                        X,Y : Integer);
begin
  if CellIndex = (Parent as TCustomPaletteX).FCellClicked then begin
   (Parent as TCustomPalettex).DoCellClick(Button,CellIndex,Color);
  end;
end;


constructor TCustomPaletteX.Create(AOwner : TComponent);
var
 i : integer;
begin
 inherited Create(AOwner);
 Width := 100;
 Height := 100;
 FColumns := columns4;
 Screen.Cursors[TheCursor] := LoadCursor(hInstance,pChar('PICKER'));
 for i := 0 to 15 do begin
  Cells[i] := TCellX.Create(self);
  Cells[i].Parent := self;
  Cells[i].CellIndex := i;
 end;
 SetStandardColors;
 Resize;
end;

procedure TCustomPaletteX.Resize;
begin
 PositionCells;
 if AdjustHeight <> 0 then Height := AdjustHeight;
 if AdjustWidth <> 0 then  Width := AdjustWidth;
 AdjustWidth := 0;
 AdjustHeight := 0;
 inherited Resize;
end;

procedure TCustomPaletteX.SetStandardColors;
var
 i : integer;
begin
 Cells[0].Color := clBlack;
 Cells[1].Color := clMaroon;
 Cells[2].Color := clGreen;
 Cells[3].Color := clOlive;
 Cells[4].Color := clNavy;
 Cells[5].Color := clPurple;
 Cells[6].Color := clTeal;
 Cells[7].Color := clGray;
 Cells[8].Color := clSilver;
 Cells[9].Color := clRed;
 Cells[10].Color := clLime;
 Cells[11].Color := clYellow;
 Cells[12].Color := clBlue;
 Cells[13].Color := clFuchsia;
 Cells[14].Color := clAqua;
 Cells[15].Color := clWhite;
 for i := 0 to 15 do begin
  Colors[i] := Cells[i].Color;
 end;
end;

procedure TCustomPaletteX.PositionCells;
var
 i : integer;
begin
  AdjustWidth := 0;
  AdjustHeight := 0;
  if FColumns = columns2 then begin
   CellWidth := Width div 2;
   CellHeight := Height div 8;
   for i := 0 to 15 do begin
    Cells[i].Width := CellWidth;
    Cells[i].Height := CellHeight;
   end;
   Cells[0].Left := 0;
   Cells[0].Top := 0;
   Cells[1].Left := CellWidth;
   Cells[1].Top := 0;
   Cells[2].Left := 0;
   Cells[2].Top := CellHeight;
   Cells[3].Left := CellWidth;
   Cells[3].Top := CellHeight;
   Cells[4].Left := 0;
   Cells[4].Top := CellHeight * 2;
   Cells[5].Left := CellWidth;
   Cells[5].Top := CellHeight * 2;
   Cells[6].Left := 0;
   Cells[6].Top := CellHeight * 3;
   Cells[7].Left := CellWidth;
   Cells[7].Top := CellHeight * 3;
   Cells[8].Left := 0;
   Cells[8].Top := CellHeight * 4;
   Cells[9].Left := CellWidth;
   Cells[9].Top := CellHeight * 4;
   Cells[10].Left := 0;
   Cells[10].Top := CellHeight * 5;
   Cells[11].Left := CellWidth;
   Cells[11].Top := CellHeight * 5;
   Cells[12].Left := 0;
   Cells[12].Top := CellHeight * 6;
   Cells[13].Left := CellWidth;
   Cells[13].Top := CellHeight * 6;
   Cells[14].Left := 0;
   Cells[14].Top := CellHeight * 7;
   Cells[15].Left := CellWidth;
   Cells[15].Top := CellHeight * 7;
   if Height > Cells[14].Top + CellHeight then begin
    AdjustHeight := Cells[14].Top + CellHeight;
   end;
  end;
  if FColumns = columns4 then begin
   CellWidth := Width div 4;
   CellHeight := Height div 4;
   for i := 0 to 15 do begin
    Cells[i].Width := CellWidth;
    Cells[i].Height := CellHeight;
   end;
   Cells[0].Left := 0;
   Cells[0].Top := 0;
   Cells[1].Left := CellWidth;
   Cells[1].Top := 0;
   Cells[2].Left := CellWidth * 2;
   Cells[2].Top := 0;
   Cells[3].Left := CellWidth * 3;
   Cells[3].Top := 0;
   Cells[4].Left := 0;
   Cells[4].Top := CellHeight;
   Cells[5].Left := CellWidth;
   Cells[5].Top := CellHeight;
   Cells[6].Left := CellWidth * 2;
   Cells[6].Top := CellHeight;
   Cells[7].Left := CellWidth * 3;
   Cells[7].Top := CellHeight;
   Cells[8].Left := 0;
   Cells[8].Top := CellHeight * 2;
   Cells[9].Left := CellWidth;
   Cells[9].Top := CellHeight * 2;
   Cells[10].Left := CellWidth * 2;
   Cells[10].Top := CellHeight * 2;
   Cells[11].Left := CellWidth * 3;
   Cells[11].Top := CellHeight * 2;
   Cells[12].Left := 0;
   Cells[12].Top := CellHeight * 3;
   Cells[13].Left := CellWidth;
   Cells[13].Top := CellHeight * 3;
   Cells[14].Left := CellWidth * 2;
   Cells[14].Top := CellHeight * 3;
   Cells[15].Left := CellWidth * 3;
   Cells[15].Top := CellHeight * 3;
  end;
  if FColumns = columns8 then begin
   CellWidth := Width div 8;
   CellHeight := Height div 2;
   for i := 0 to 15 do begin
    Cells[i].Width := CellWidth;
    Cells[i].Height := CellHeight;
   end;
   Cells[0].Left := 0;
   Cells[0].Top := 0;
   Cells[1].Left := CellWidth;
   Cells[1].Top := 0;
   Cells[2].Left := CellWidth * 2;
   Cells[2].Top := 0;
   Cells[3].Left := CellWidth * 3;
   Cells[3].Top := 0;
   Cells[4].Left := CellWidth * 4;
   Cells[4].Top := 0;
   Cells[5].Left := CellWidth * 5;
   Cells[5].Top := 0;
   Cells[6].Left := CellWidth * 6;
   Cells[6].Top := 0;
   Cells[7].Left := CellWidth * 7;
   Cells[7].Top := 0;
   Cells[8].Left := 0;
   Cells[8].Top := CellHeight;
   Cells[9].Left := CellWidth;
   Cells[9].Top := CellHeight;
   Cells[10].Left := CellWidth * 2;
   Cells[10].Top := CellHeight;
   Cells[11].Left := CellWidth * 3;
   Cells[11].Top := CellHeight;
   Cells[12].Left := CellWidth * 4;
   Cells[12].Top := CellHeight;
   Cells[13].Left := CellWidth * 5;
   Cells[13].Top := CellHeight;
   Cells[14].Left := CellWidth * 6;
   Cells[14].Top := CellHeight;
   Cells[15].Left := CellWidth * 7;
   Cells[15].Top := CellHeight;
   if Width > Cells[7].Left + CellWidth then begin
    AdjustWidth := Cells[7].Left + CellWidth;
   end;
  end;
end;



procedure TCustomPaletteX.SetColumns(Num : TColumnsX);
begin
 FColumns := Num;
 Resize;
end;

procedure TCustomPaletteX.SetCellColor(index : integer;Color : TColor);
begin
 Colors[index] := Color;
 Cells[index].Color := Color;
end;

function TCustomPaletteX.GetCellColor(index : integer) : TColor;
begin
 result := Cells[index].Color;
end;

procedure TCustomPaletteX.DoCellClick(Button : TMouseButton;
                                     Index : Integer;Color : TColor);
begin
  if Assigned(FOnCellClick) then
   FOnCellClick(Index,Button,Color);
end;

procedure TCustomPaletteX.SaveToFile(name : String);
begin
 IniFile := TIniFile.Create(name);
 IniFile.WriteInteger('Colors','Color00',Cells[0].Color);
 IniFile.WriteInteger('Colors','Color01',Cells[1].Color);
 IniFile.WriteInteger('Colors','Color02',Cells[2].Color);
 IniFile.WriteInteger('Colors','Color03',Cells[3].Color);
 IniFile.WriteInteger('Colors','Color04',Cells[4].Color);
 IniFile.WriteInteger('Colors','Color05',Cells[5].Color);
 IniFile.WriteInteger('Colors','Color06',Cells[6].Color);
 IniFile.WriteInteger('Colors','Color07',Cells[7].Color);
 IniFile.WriteInteger('Colors','Color08',Cells[8].Color);
 IniFile.WriteInteger('Colors','Color09',Cells[9].Color);
 IniFile.WriteInteger('Colors','Color10',Cells[10].Color);
 IniFile.WriteInteger('Colors','Color11',Cells[11].Color);
 IniFile.WriteInteger('Colors','Color12',Cells[12].Color);
 IniFile.WriteInteger('Colors','Color13',Cells[13].Color);
 IniFile.WriteInteger('Colors','Color14',Cells[14].Color);
 IniFile.WriteInteger('Colors','Color15',Cells[15].Color);
 IniFile.Free;
end;

procedure TCustomPaletteX.LoadFromFile(name : String);
begin
  if not FileExists(name) then exit;
  IniFile := TIniFile.Create(name);
  Cells[0].Color := IniFile.ReadInteger('Colors','Color00',0);
  Cells[1].Color := IniFile.ReadInteger('Colors','Color01',0);
  Cells[2].Color := IniFile.ReadInteger('Colors','Color02',0);
  Cells[3].Color := IniFile.ReadInteger('Colors','Color03',0);
  Cells[4].Color := IniFile.ReadInteger('Colors','Color04',0);
  Cells[5].Color := IniFile.ReadInteger('Colors','Color05',0);
  Cells[6].Color := IniFile.ReadInteger('Colors','Color06',0);
  Cells[7].Color := IniFile.ReadInteger('Colors','Color07',0);
  Cells[8].Color := IniFile.ReadInteger('Colors','Color08',0);
  Cells[9].Color := IniFile.ReadInteger('Colors','Color09',0);
  Cells[10].Color := IniFile.ReadInteger('Colors','Color10',0);
  Cells[11].Color := IniFile.ReadInteger('Colors','Color11',0);
  Cells[12].Color := IniFile.ReadInteger('Colors','Color12',0);
  Cells[13].Color := IniFile.ReadInteger('Colors','Color13',0);
  Cells[14].Color := IniFile.ReadInteger('Colors','Color14',0);
  Cells[15].Color := IniFile.ReadInteger('Colors','Color15',0);
  IniFile.Free;
end;



//----- TColorPanelX Code -------------------------------------------------


constructor TColorPanelX.Create(AOwner : TComponent);
begin
 inherited Create(Aowner);
 Width := 80;
 Height := 101;
 BevelInner:=bvLowered;
 BevelOuter:=bvRaised;
 BorderStyle := bsSingle;

 Image := TImage.Create(self);
 Image.Parent := self;
 Image.Align := alClient;
 Image.Cursor := TheCursor;
 Image.Picture.Bitmap.LoadFromResourceName(hInstance,'CCLRPANELX');
 Image.OnMouseDown := DoSelect;
 Image.OnMouseMove := DoChange;
end;

procedure TColorPanelX.DoSelect(Sender: TObject;Button: TMouseButton;
                       Shift: TShiftState; X, Y: Integer);
begin
  ColorSelected := Image.Canvas.Pixels[x,y];
  if Button = mbLeft then begin
   if Assigned(FLeftSelect) then FLeftSelect(ColorSelected);
  end;
  if Button = mbRight then begin
   if Assigned(FRightSelect) then FRightSelect(ColorSelected);
  end;
end;

procedure TColorPanelX.DoChange(Sender: TObject; Shift: TShiftState;X, Y: Integer);
begin
 ColorSelected := Image.Canvas.Pixels[x,y];
 if Assigned(FChange) then FChange(ColorSelected);
end;



//----- Main Component Code -----------------------------------------------

constructor TColorPicker.Create(AOwner : TComponent);
begin
 inherited Create(Aowner);
 Screen.Cursors[TheCursor] := LoadCursor(hInstance, pChar('CPICKERX'));
 Width := 182;
 Height := 240;
 BevelInner:=bvLowered;
 BevelOuter:=bvRaised;
 BorderStyle := bsSingle;

 Back := TPanel.Create(self);
 Back.Parent := self;
 Back.Align := alClient;

 Current := TPanel.Create(self);
 Current.Parent := self;
 Current.Left := 5;
 Current.Top := 8;
 Current.Width := 80;
 Current.Height := 35;
 Current.Color := clBlack;
 Current.BevelInner:=bvLowered;
 Current.BevelOuter:=bvRaised;
 Current.BorderStyle := bsSingle;

 ColorBox := TColorPanelX.Create(self);
 ColorBox.Parent := self;
 ColorBox.Left := 5;
 ColorBox.Top := 45;
 ColorBox.OnLeftSelect := PrimaryColorChange;
 ColorBox.OnRightSelect := SecondaryColorChange;
 ColorBox.OnNewColor := CurrentChange;

 Group1 := TGroupBox.Create(self);
 Group1.Parent := self;
 Group1.Left := 90;
 Group1.Top := 3;
 Group1.Width := 80;
 Group1.Height := 65;
 Group1.Caption := 'Primary';

 Group2 := TGroupBox.Create(self);
 Group2.Parent := self;
 Group2.Left := 90;
 Group2.Top := 77;
 Group2.Width := 80;
 Group2.Height := 68;
 Group2.Caption := 'Secondary';

 Prime := TPanel.Create(self);
 Prime.Parent := Group1;
 Prime.Left := 10;
 Prime.Top := 16;
 Prime.Width := 60;
 Prime.Height := 40;
 Prime.BevelInner:=bvLowered;
 Prime.BevelOuter:=bvRaised;
 Prime.BorderStyle := bsSingle;

 Second := TPanel.Create(self);
 Second.Parent := Group2;
 Second.Left := 10;
 Second.Top := 18;
 Second.Width := 60;
 Second.Height := 40;
 Second.BevelInner:=bvLowered;
 Second.BevelOuter:=bvRaised;
 Second.BorderStyle := bsSingle;

 Palette := TCustomPaletteX.Create(self);
 Palette.Parent := self;
 Palette.Top := 150;
 Palette.Left := 6;
 Palette.Width := 80;
 Palette.Height := 80;
 Palette.OnCellClick := OnPaletteClick;

 BtnSave := TButton.Create(self);
 BtnSave.Parent := self;
 BtnSave.Top := 152;
 BtnSave.Left := 90;
 BtnSave.Width := 80;
 BtnSave.Height := 35;
 BtnSave.Caption := 'Save Palette';
 BtnSave.OnClick := ButtonClick;

 BtnLoad := TButton.Create(self);
 BtnLoad.Parent := self;
 BtnLoad.Top := 192;
 BtnLoad.Left := 90;
 BtnLoad.Width := 80;
 BtnLoad.Height := 35;
 BtnLoad.Caption := 'Load Palette';
 BtnLoad.OnClick := ButtonClick;

 PrimaryColorChange(clBlack);
 SecondaryColorChange(clWhite);
end;

procedure TColorPicker.Resize;
begin
 if Width <> 182 then Width := 182;
 if Height <> 240 then Height := 240;
 inherited Resize;
end;

procedure TColorPicker.PrimaryColorChange(aColor : TColor);
begin
 FPrimeColor := aColor;
 Prime.Color := aColor;
 if Assigned(NewPrime) then NewPrime;
end;

procedure TColorPicker.SecondaryColorChange(aColor : TColor);
begin
 FSecondColor := aColor;
 Second.Color := aColor;
 if Assigned(NewSecond) then NewSecond;
end;

procedure TColorPicker.CurrentChange(aColor : TColor);
begin
 Current.Color := aColor;
end;

procedure TColorPicker.OnPaletteClick(Index :Integer;
                                       Button : TMouseButton;Color : TColor);
begin
 if Button = mbLeft then PrimaryColorChange(Color);
 if Button = mbRight then begin
  Palette.SetCellColor(Index,Prime.Color);
  if Assigned(NewPalette) then NewPalette(Index,Prime.Color);
 end;
end;

procedure TColorPicker.SavePalette(name : string);
begin
 Palette.SaveToFile(name);
end;

procedure TColorPicker.LoadPalette(name : string);
begin
 Palette.LoadFromFile(name);
end;

procedure TColorPicker.ButtonClick(Sender : TObject);
begin
   if (Sender as TButton) = BtnSave then begin
     if Assigned(FSaveSelect) then FSaveSelect(Sender);
   end;
   if (Sender as TButton) = BtnLoad then begin
    if Assigned(FLoadSelect) then FloadSelect(Sender);
   end;
end;

Function  TColorPicker.GetPaletteColor(Index : Integer) : TColor;
begin
  result := Palette.GetCellColor(Index);
end;

Procedure TColorPicker.SetPaletteColor(Index : Integer;aColor : TColor);
begin
  Palette.SetCellColor(Index,aColor);
end;

Procedure TColorPicker.SetStandardPalette;
begin
 Palette.SetStandardColors;
end;

end.
